﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.BusinessLogicsContracts;
using Microsoft.Extensions.Logging;
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;

namespace GarmentFactoryView
{
    public partial class FormReportShops : Form
    {
        private readonly ILogger _logger;
        private readonly IReportLogic _logic;

        public FormReportShops(ILogger<FormReportShops> logger, IReportLogic logic)
        {
            InitializeComponent();
            _logger = logger;
            _logic = logic;
        }

        private void FormReportShops_Load(object sender, EventArgs e)
        {
            try
            {
                var shops = _logic.GetShops();
                if (shops != null)
                {
                    dataGridView.Rows.Clear();
                    foreach (var shop in shops)
                    {
                        dataGridView.Rows.Add(new object[] { shop.ShopName, "", "" });
                        foreach (var textile in shop.Textiles)
                        {
                            dataGridView.Rows.Add(new object[] { "", textile.Item1, textile.Item2 });
                        }
                        dataGridView.Rows.Add(new object[] { "Итого", "", shop.TotalCount });
                        dataGridView.Rows.Add(Array.Empty<object>());
                    }
                }
                _logger.LogInformation("Загрузка списка текстилей по магазинам");
            }
            catch (Exception ex)
            {
                _logger.LogError(ex, "Ошибка загрузки списка текстилей по магазинам");
                MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
            }
        }

        private void ButtonSaveToExcel_Click(object sender, EventArgs e)
        {
            using var dialog = new SaveFileDialog { Filter = "xlsx|*.xlsx" };
            if (dialog.ShowDialog() == DialogResult.OK)
            {
                try
                {
                    _logic.SaveShopsToExcelFile(new ReportBindingModel
                    {
                        FileName = dialog.FileName
                    });
                    _logger.LogInformation("Сохранение списка текстилей по магазинам");
                    MessageBox.Show("Выполнено", "Успех", MessageBoxButtons.OK, MessageBoxIcon.Information);
                }
                catch (Exception ex)
                {
                    _logger.LogError(ex, "Ошибка сохранения списка текстилей по магазинам");
                    MessageBox.Show(ex.Message, "Ошибка", MessageBoxButtons.OK, MessageBoxIcon.Error);
                }
            }
        }
    }
}
