﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.ComponentModel.DataAnnotations.Schema;
using System.ComponentModel.DataAnnotations;
using GarmentFactoryDataModels.Models;
using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.ViewModels;

namespace GarmentFactoryDatabaseImplement.Models
{
	public class Shop : IShopModel
	{
		public int Id { get; set; }

		[Required]
		public string ShopName { get; set; } = string.Empty;

		[Required]
		public string Address { get; set; } = string.Empty;

		[Required]
		public DateTime DateOpen { get; set; }

		[Required]
		public int TextileMaxCount { get; set; }

		[ForeignKey("ShopId")]
		public List<ShopTextile> Textiles { get; set; } = new();

		private Dictionary<int, (ITextileModel, int)>? _shopTextiles = null;

		[NotMapped]
		public Dictionary<int, (ITextileModel, int)> ShopTextiles
		{
			get
			{
				if (_shopTextiles == null)
				{
					if (_shopTextiles == null)
					{
						_shopTextiles = Textiles.ToDictionary(x => x.TextileId, y => (y.Textile as ITextileModel, y.Count));
					}
					return _shopTextiles;
				}

				return _shopTextiles;
			}
		}

		public static Shop Create(GarmentFactoryDatabase context, ShopBindingModel model)
		{
			return new Shop()
			{
				Id = model.Id,
				ShopName = model.ShopName,
				Address = model.Address,
				DateOpen = model.DateOpen,
				TextileMaxCount = model.TextileMaxCount,
				Textiles = model.ShopTextiles.Select(x => new ShopTextile
				{
					Textile = context.Textiles.First(y => y.Id == x.Key),
					Count = x.Value.Item2
				}).ToList(),
			};
		}

		public void Update(ShopBindingModel model)
		{
			ShopName = model.ShopName;
			Address = model.Address;
			DateOpen = model.DateOpen;
			TextileMaxCount = model.TextileMaxCount;
		}

		public ShopViewModel GetViewModel => new()
		{
			Id = Id,
			ShopName = ShopName,
			Address = Address,
			DateOpen = DateOpen,
			ShopTextiles = ShopTextiles,
			TextileMaxCount = TextileMaxCount
		};

		public void UpdateTextiles(GarmentFactoryDatabase context, ShopBindingModel model)
		{
			var ShopTextiles = context.ShopTextiles
				.Where(rec => rec.ShopId == model.Id)
				.ToList();

			if (ShopTextiles != null && ShopTextiles.Count > 0)
			{
				context.ShopTextiles.RemoveRange(ShopTextiles.Where(rec => !model.ShopTextiles.ContainsKey(rec.TextileId)));
				context.SaveChanges();

				ShopTextiles = context.ShopTextiles.Where(rec => rec.ShopId == model.Id).ToList();

				foreach (var textileToUpdate in ShopTextiles)
				{
					textileToUpdate.Count = model.ShopTextiles[textileToUpdate.TextileId].Item2;
					model.ShopTextiles.Remove(textileToUpdate.TextileId);
				}

				context.SaveChanges();
			}

			var Shop = context.Shops.First(x => x.Id == Id);

			foreach (var ShopTextile in model.ShopTextiles)
			{
				context.ShopTextiles.Add(new ShopTextile
				{
					Shop = Shop,
					Textile = context.Textiles.First(x => x.Id == ShopTextile.Key),
					Count = ShopTextile.Value.Item2
				});

				context.SaveChanges();
			}

			_shopTextiles = null;
		}

		public void DictionaryTextilesUpdate(GarmentFactoryDatabase context)
		{
			UpdateTextiles(context, new ShopBindingModel
			{
				Id = Id,
				ShopTextiles = ShopTextiles
			});
		}
	}
}
