﻿using GarmentFactoryContracts.BindingModels;
using GarmentFactoryContracts.ViewModels;
using GarmentFactoryDataModels.Models;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Xml.Linq;

namespace GarmentFactoryFileImplement.Models
{
	public class Shop : IShopModel
	{
		public int Id { get; private set; }
		public string ShopName { get; private set; } = string.Empty;
		public string Address { get; private set; } = string.Empty;
		public DateTime DateOpen { get; private set; }
		public Dictionary<int, int> Textiles { get; private set; } = new();
		private Dictionary<int, (ITextileModel, int)>? _shopTextiles = null;

		public Dictionary<int, (ITextileModel, int)> ShopTextiles
		{
			get
			{
				if (_shopTextiles == null)
				{
					var source = DataFileSingleton.GetInstance();
					_shopTextiles = Textiles.ToDictionary(x => x.Key, y => ((source.Textiles.FirstOrDefault(z => z.Id == y.Key) as ITextileModel)!, y.Value));
				}
				return _shopTextiles;
			}
		}

		public int TextileMaxCount { get; private set; }

		public static Shop? Create(ShopBindingModel? model)
		{
			if (model == null)
			{
				return null;
			}
			return new Shop()
			{
				Id = model.Id,
				ShopName = model.ShopName,
				Address = model.Address,
				DateOpen = model.DateOpen,
				Textiles = model.ShopTextiles.ToDictionary(x => x.Key, x => x.Value.Item2),
				TextileMaxCount = model.TextileMaxCount
			};
		}

		public static Shop? Create(XElement element)
		{
			if (element == null)
			{
				return null;
			}
			return new()
			{
				Id = Convert.ToInt32(element.Attribute("Id")!.Value),
				ShopName = element.Element("ShopName")!.Value,
				Address = element.Element("Address")!.Value,
				DateOpen = Convert.ToDateTime(element.Element("DateOpen")!.Value),
				Textiles = element.Element("ShopTextiles")!.Elements("ShopTextile")!.ToDictionary(x => Convert.ToInt32(x.Element("Key")?.Value),
				x => Convert.ToInt32(x.Element("Value")?.Value)),
				TextileMaxCount = Convert.ToInt32(element.Element("TextileMaxCount")!.Value)
			};
		}

		public void Update(ShopBindingModel? model)
		{
			if (model == null)
			{
				return;
			}
			ShopName = model.ShopName;
			Address = model.Address;
			DateOpen = model.DateOpen;
			TextileMaxCount = model.TextileMaxCount;
			Textiles = model.ShopTextiles.ToDictionary(x => x.Key, x => x.Value.Item2);
			_shopTextiles = null;
		}

		public ShopViewModel GetViewModel => new()
		{
			Id = Id,
			ShopName = ShopName,
			Address = Address,
			DateOpen = DateOpen,
			ShopTextiles = ShopTextiles,
			TextileMaxCount = TextileMaxCount
		};

		public XElement GetXElement => new("Shop",
			new XAttribute("Id", Id),
			new XElement("ShopName", ShopName),
			new XElement("Address", Address),
			new XElement("DateOpen", DateOpen.ToString()),
			new XElement("ShopTextiles", Textiles.Select(
				x => new XElement("ShopTextile", new XElement("Key", x.Key), new XElement("Value", x.Value))).ToArray()),
			new XElement("TextileMaxCount", TextileMaxCount.ToString())
			);

		public void TextilesUpdate()
		{
			_shopTextiles = null;
		}
	}
}
